{$X+ discardable function returns }
PROGRAM MatrixJuggler ;

{
	MJ - The Matrix Juggler.
	Copyright  1992  TCastle Software

	Thanks to Paul E Hess for the inspiration
	for this program.
}

USES
		BatchErr
	,	Dos
	,	InputObj
	,	MJO
	,	TCastle
	;

{$I FLOAT.INC }

TYPE
	PMatrixProgram	= ^TMatrixProgram ;
	TMatrixProgram	= OBJECT ( TProteus )
		pMJO	: PMatrixJugglerObject ;
		CONSTRUCTOR Init ;
		DESTRUCTOR	Done ; VIRTUAL ;
		PROCEDURE	AddMatrices ;
		PROCEDURE	CopyMatrix ;
		PROCEDURE	EigenExtract ;
		PROCEDURE	HandleKeyWord ( VAR aKeyWord : KeyWordType ) ; VIRTUAL ;
		PROCEDURE	InvertMatrix ;
		PROCEDURE	MultiplyMatrices ;
		PROCEDURE	PrintMatrix ;
      PROCEDURE	PrintTheCollection ;
		PROCEDURE	ReadMatrix;
		PROCEDURE	Run ; VIRTUAL ;
		PROCEDURE	SubtractMatrices ;
		PROCEDURE	TransposeMatrix ;
	END ;


{  -----  TMatrixProgram Methods  -----  }


CONSTRUCTOR	TMatrixProgram.Init ;
	CONST
		mon : ARRAY [ 1 .. 12 ] OF STRING[3]
			= (	'Jan' , 'Feb' , 'Mar' , 'Apr' , 'May' , 'Jun' ,
					'Jul' , 'Aug' , 'Sep' , 'Oct' , 'Nov' , 'Dec' ) ;
		dow : ARRAY [ 0 .. 6 ] OF STRING[9]
			= (	'Sunday' , 'Monday' , 'Tuesday' , 'Wednesday' , 'Thursday' ,
					'Friday' , 'Saturday' ) ;
	VAR
		DirInfo : SearchRec;
		IFileName : PathStr;
		OFileName : PathStr;
		Dir : DirStr;
		Name: NameStr;
		Ext : ExtStr ;
		hour, minute, second, sec100 : WORD ;
		year, month, day, dayofweek : WORD ;
	BEGIN					{  Init  }
		pError := New( PBatchErrorObject, Init );
		pError^.WriteLine( ' ' ) ;
		pError^.WriteLine( 'The Matrix Juggler' ) ;
		pError^.WriteLine( 'Batch version via TProteus '+version ) ;
		pError^.WriteLine( 'Copyright  1992  TCastle Software' ) ;
		pError^.WriteLine( ' ' ) ;
		{  check command line for input file  }
		IF  ParamCount = 1  THEN BEGIN
			IFileName := FExpand(ParamStr(1));
			FindFirst(IFileName, Archive, DirInfo) ;
			IF  DosError <> 0  THEN BEGIN
				pError^.Fatal( 'Trouble with '+IFileName )
			END ;
			IF  NOT  InitInput( IFileName )  THEN BEGIN
				pError^.Fatal( 'Trouble with '+IFileName )
			END ;
			FSplit( IFileName, Dir, Name, Ext ) ;
			OFileName := Dir + Name + '.OUT' ;
			IF  NOT InitOutput( OFileName )  THEN BEGIN
				pError^.Fatal( 'Trouble with '+OFileName )
			END ;
		END
		ELSE BEGIN
			pError^.Inform( 'usage: '+ParamStr(0)+'  input' ) ;
			Halt( 1 )
		END ;
		Writeln( FOut, 'The Matrix Juggler' ) ;
		Writeln( FOut, 'Batch version via TProteus ', version ) ;
		Writeln( FOut, 'Copyright  1992  TCastle Software' ) ;
		GetDate( year, month, day, dayofweek ) ;
		GetTime( hour, minute, second, sec100 ) ;
		Writeln( FOut ) ;
		Write( FOut, dow[dayofweek], '  ', day, ' ', mon[month], ' ', year ) ;
		Write( FOut, '   ' ) ;
		IF  hour > 12  THEN Write( FOut, hour - 12 )
					   ELSE Write( FOut, hour ) ;
		Write( FOut, ':' ) ;
		IF  minute < 10 THEN Write( FOut, '0', minute )
						ELSE Write( FOut, minute ) ;
		IF  hour <= 12 THEN Write( FOut, 'am.' )
						ELSE Write( FOut, 'pm.' ) ;
		Writeln( FOut ) ;
		Writeln( FOut ) ;
		Writeln( FOut, 'Input file:  ', IFileName ) ;
		pMJO := New( PMatrixJugglerObject, Init ) ;
	END ;					{  Init  }


DESTRUCTOR	TMatrixProgram.Done ;	{  VIRTUAL  }
	BEGIN					{  Done  }
		Dispose( pMJO, Done ) ;
		TProteus.Done ;
	END ;					{  Done  }


PROCEDURE	TMatrixProgram.AddMatrices ;
	BEGIN					{  AddMatrices  }
   	pMJO^.AddMatrices ;
	END ;					{  AddMatrices  }


PROCEDURE	TMatrixProgram.CopyMatrix ;
	BEGIN					{  CopyMatrix  }
		pMJO^.CopyMatrix ;
	END ;					{  CopyMatrix  }


PROCEDURE	TMatrixProgram.EigenExtract ;
	BEGIN					{  EigenExtract  }
		pMJO^.EigenExtract ;
	END ;					{  EigenExtract  }


PROCEDURE	TMatrixProgram.HandleKeyWord(VAR aKeyWord : KeyWordType); {VIRTUAL}
	BEGIN					{  HandleKeyWord  }
		TProteus.HandleKeyWord( aKeyWord ) ;
		Writeln( FOut ) ;
		Writeln( FOut, pInput^.RawLineIs ) ;	{  echo the non-blank lines  }
		WITH  aKeyWord  DO BEGIN
			IF  What = 'ADD'  THEN BEGIN
				AddMatrices ;
			END
			ELSE IF  What = 'COPY'  THEN BEGIN
				CopyMatrix ;
			END
			ELSE IF  What = 'EIGEN'  THEN BEGIN
				EigenExtract ;
			END
			ELSE IF  What = 'INVERT'  THEN BEGIN
				InvertMatrix ;
			END
			ELSE IF  What = 'MULTIPLY'  THEN BEGIN
				MultiplyMatrices ;
			END
			ELSE IF  What = 'PRINT'  THEN BEGIN
         	PrintMatrix ;
			END
			ELSE IF  What = 'PRINTALL'  THEN BEGIN
         	PrintTheCollection ;
			END
			ELSE IF  What = 'READ'  THEN BEGIN
				ReadMatrix;
			END
			ELSE IF  What = 'SUBTRACT'  THEN BEGIN
				SubtractMatrices ;
			END
			ELSE IF  What = 'TRANSPOSE'  THEN BEGIN
				TransposeMatrix ;
			END
			ELSE BEGIN
				Exit
			END ;
			ClearKeyWord( aKeyWord )
		END
	END ;					{  HandleKeyWord  }


PROCEDURE	TMatrixProgram.InvertMatrix ;
	BEGIN					{  InvertMatrix  }
		pMJO^.InvertMatrix ;
	END ;					{  InvertMatrix  }


PROCEDURE	TMatrixProgram.MultiplyMatrices ;
	BEGIN					{  MultiplyMatrices  }
		pMJO^.MultiplyMatrices ;
	END ;					{  MultiplyMatrices  }


PROCEDURE	TMatrixProgram.PrintMatrix ;
	BEGIN					{  PrintMatrix  }
		pMJO^.PrintMatrix ;
	END ;					{  PrintMatrix  }


PROCEDURE	TMatrixProgram.PrintTheCollection ;
	BEGIN					{  PrintTheCollection  }
		pMJO^.PrintTheCollection ;
	END ;					{  PrintTheCollection  }


PROCEDURE	TMatrixProgram.ReadMatrix;
	BEGIN					{  ReadMatrix  }
   	pMJO^.ReadMatrix ;
	END ;					{  ReadMatrix  }


PROCEDURE	TMatrixProgram.Run ;	{  VIRTUAL  }
	BEGIN					{  Run  }
   	ReadInput ;
	END ;					{  Run  }


PROCEDURE	TMatrixProgram.SubtractMatrices ;
	BEGIN					{  SubtractMatrices  }
		pMJO^.SubtractMatrices ;
	END ;					{  SubtractMatrices  }


PROCEDURE	TMatrixProgram.TransposeMatrix ;
	BEGIN					{  TransposeMatrix  }
		pMJO^.TransposeMatrix ;
	END ;					{  TransposeMatrix  }


VAR
	aProgram	: TMatrixProgram ;
	m1, m2	: LONGINT ;


BEGIN						{  ----- main program -----  }
	m1 := MaxAvail ;
	aProgram.Init  ;
	aProgram.Run ;
	aProgram.Done ;
	m2 := MaxAvail ;
	IF  m1 <> m2  THEN BEGIN
		Writeln( 'Leaky Memory.' );
		Writeln( 'At start:', m1:10 );
		Writeln( 'At end:  ', m2:10 );
		Writeln( 'Missing: ', m2-m1:10 ) ;
	END ;
END .
