{$X+ discard function return }
UNIT	MJO ;


	{]=================================================[]
	||                                                 ||
	||             TMatrixJugglerObject                ||
	||                                                 ||
	[]=================================================[]
	||                                                 ||
	||        Copyright  1992  TCastle Software        ||
	||                                                 ||
	||                                            ||
	||                                              ||
	||             TCastle  Software              ||
	||                                                 ||
	[]=================================================[}


INTERFACE


USES
		BatchErr
	,	InputObj
	,	Matrices
	,	NamedMat
	,	Objects
	,	TCastle
	;

{$I FLOAT.INC }


CONST
	TMatrixJugglerObjectID	= 60004 ;	{  Stream Registry  }


TYPE
	PMatrixJugglerObject	= ^TMatrixJugglerObject ;
	TMatrixJugglerObject	= OBJECT ( TNamedMatrix )
		CONSTRUCTOR Init ;
		DESTRUCTOR	Done ; VIRTUAL ;
		PROCEDURE	AddMatrices ;
		PROCEDURE	CopyMatrix ;
      PROCEDURE	EigenExtract ;
		PROCEDURE	ExistError( aName, aMethodName : STRING ) ;
		FUNCTION		FindMatrix ( aName : STRING ; VAR index : INTEGER ) : PNamedMatrix ;
		PROCEDURE	InvertMatrix ;
		PROCEDURE	MultiplyMatrices ;
		PROCEDURE	PrintMatrix ;
      PROCEDURE	PrintTheCollection ;
		PROCEDURE	ReadMatrix;
		PROCEDURE	ShapeError( aMethodName : STRING ) ;
		PROCEDURE	SubtractMatrices ;
		PROCEDURE	SyntaxError( ExpectedLine, aMethodName : STRING ) ;
		PROCEDURE	TransposeMatrix;
	END ;


PROCEDURE	RegisterTMatrixJugglerObject ;


IMPLEMENTATION


USES
		Utility
	;


CONST
	RMatrixJugglerObject: TStreamRec = (
		ObjType: TMatrixJugglerObjectID ;
		VmtLink: Ofs(TypeOf(TMatrixJugglerObject)^);
		Load:    @TMatrixJugglerObject.Load;
		Store:   @TMatrixJugglerObject.Store
	);


PROCEDURE	RegisterTMatrixJugglerObject ;
	BEGIN					{  RegisterTMatrixJugglerObject  }
		RegisterType( RMatrixJugglerObject ) ;
		RegisterTNamedMatrix ;
	END ;					{  RegisterTMatrixJugglerObject  }


{  -----  TMatrixJugglerObject Methods  -----  }


CONSTRUCTOR	TMatrixJugglerObject.Init ;
	BEGIN					{  Init  }
		pMatrices := New( PNamedMatrixCollection, Init( 20, 20 )) ;
	END ;					{  Init  }


DESTRUCTOR	TMatrixJugglerObject.Done ;	{  VIRTUAL  }
	BEGIN					{  Done  }
		Dispose( pMatrices, Done ) ;
	END ;					{  Done  }


PROCEDURE	TMatrixJugglerObject.AddMatrices ;
	CONST
		ExpectedLine	= 'Expected line:  ADD  Augend  Addend  Sum' ;
      MethodName		= 'AddMatrices' ;
	VAR
		iA, iB, iC	: INTEGER;
		nra,nca	: WORD ;
		pAugend	: PNamedMatrix ;
		pAddend	: PNamedMatrix ;
		pSum		: PNamedMatrix ;
		sAugend	: STRING ;
		sAddend	: STRING ;
		sSum		: STRING ;
	BEGIN					{  AddMatrices  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( sAugend ) = '')
			OR  (GetToken( sAddend ) = '')
			OR  (GetToken( sSum ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  augend exists?  }
		pAugend	:= FindMatrix( sAugend, iA ) ;
		IF  pAugend = NIL  THEN BEGIN
			ExistError( sAugend, MethodName ) ;
		END ;
		nra	:= pAugend^.Rows ;
		nca	:= pAugend^.Cols ;
							{  addend exists?  }
		pAddend	:= FindMatrix( sAddend, iB ) ;
		IF  pAddend = NIL  THEN BEGIN
			ExistError( sAddend, MethodName ) ;
		END ;
							{  if sum exists, trash it  }
		pSum	:= FindMatrix( sSum, iC ) ;
		IF  pSum <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( pSum, Done ) ;
		END;
		pSum := New( PNamedMatrix, Init( sSum, nra, nca )) ;
		pAugend^.Add( pAddend^, pSum^ ) ;
		IF  pAugend^.ErrorStatusIs <> 0  THEN BEGIN
			ShapeError( MethodName ) ;
		END ;
		pMatrices^.AtInsert( IC, pSum ) ;
	END ;					{  AddMatrices  }


PROCEDURE	TMatrixJugglerObject.CopyMatrix ;
	CONST
		ExpectedLine	= 'Expected line:  Copy Source Target' ;
		MethodName		= 'CopyMatrix' ;
	VAR
		iA, iC	: INTEGER;
		nra,nca	: WORD ;
		pSource	: PNamedMatrix ;
		pTarget	: PNamedMatrix ;
		sSource	: STRING ;
		sTarget	: STRING ;
	BEGIN					{  CopyMatrix  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( sSource ) = '')
			OR  (GetToken( sTarget ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  Source exists?  }
		pSource	:= FindMatrix( sSource, iA ) ;
		IF  pSource = NIL  THEN BEGIN
			ExistError( sSource, MethodName ) ;
		END ;
		nra	:= pSource^.Rows ;
		nca	:= pSource^.Cols ;
							{  if Target exists, trash it  }
		pTarget	:= FindMatrix( sTarget, iC ) ;
		IF  pTarget <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( pTarget, Done ) ;
		END;
		pTarget := New( PNamedMatrix, Init( sTarget, nra, nca )) ;
      pTarget^.Copy( pSource^ ) ;
		pMatrices^.AtInsert( IC, pTarget ) ;
	END ;					{  CopyMatrix  }


PROCEDURE	TMatrixJugglerObject.EigenExtract ;
	CONST
		ExpectedLine	= 'Expected line:  Eigen  aMatrix  eValues  eVectors' ;
		MethodName		= 'EigenExtract' ;
	VAR
		iA, iB, iC	: INTEGER;
		nra,nca	: WORD ;
		paMatrix	: PNamedMatrix ;
		peValues	: PNamedMatrix ;
		peVectors		: PNamedMatrix ;
		saMatrix	: STRING ;
		seValues	: STRING ;
		seVectors		: STRING ;
	BEGIN					{  EigenExtract  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( saMatrix ) = '')
			OR  (GetToken( seValues ) = '')
			OR  (GetToken( seVectors ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  aMatrix exists?  }
		paMatrix	:= FindMatrix( saMatrix, iA ) ;
		IF  paMatrix = NIL  THEN BEGIN
			ExistError( saMatrix, MethodName ) ;
		END ;
		nra	:= paMatrix^.Rows ;
		nca	:= paMatrix^.Cols ;
							{  if eValues exists, trash it  }
		peValues	:= FindMatrix( seValues, iB ) ;
		IF  peValues <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iB ) ;
			Dispose( peValues, Done ) ;
		END;
		peValues := New( PNamedMatrix, Init( seValues, nra, 1 )) ;
							{  if eVectors exists, trash it  }
		peVectors	:= FindMatrix( seVectors, iC ) ;
		IF  peVectors <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( peVectors, Done ) ;
		END;
		peVectors := New( PNamedMatrix, Init( seVectors, nra, nca )) ;
		paMatrix^.Jacobi( peValues^, peVectors^ ) ;
		IF  paMatrix^.ErrorStatusIs <> 0  THEN BEGIN
			ShapeError( MethodName ) ;
		END ;
		pMatrices^.AtInsert( IB, peValues ) ;
		pMatrices^.AtInsert( IC, peVectors ) ;
	END ;					{  EigenExtract  }


PROCEDURE	TMatrixJugglerObject.ExistError( aName, aMethodName : STRING ) ;
	BEGIN					{  ExistError  }
		pError^.WriteLine( pInput^.EchoLine );
		pError^.Fatal( aName + ' doesn''t exist. - ' + aMethodName ) ;
	END ;					{  ExistError  }


FUNCTION		TMatrixJugglerObject.FindMatrix ( aName : STRING ; VAR index : INTEGER ) : PNamedMatrix ;
	VAR
		pS			: PString ;
	BEGIN					{  FindMatrix  }
		pS := NewStr( aName ) ;
		IF  pMatrices^.Search( pS, index )  THEN BEGIN
			FindMatrix	:= pMatrices^.At( index ) ;
		END
		ELSE BEGIN
      	FindMatrix	:= NIL ;
		END ;
		DisposeStr( pS ) ;
	END ;					{  FindMatrix  }


PROCEDURE	TMatrixJugglerObject.InvertMatrix ;
	CONST
		ExpectedLine	= 'Expected line:  Invert aMatrix' ;
		MethodName		= 'InvertMatrix' ;
	VAR
   	Determ	: FLOAT ;
		iA	: INTEGER;
		paMatrix	: PNamedMatrix ;
		saMatrix	: STRING ;
	BEGIN					{  InvertMatrix  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( saMatrix ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  aMatrix exists?  }
		paMatrix	:= FindMatrix( saMatrix, iA ) ;
		IF  paMatrix = NIL  THEN BEGIN
			ExistError( saMatrix, MethodName ) ;
		END ;
		IF  NOT paMatrix^.Invert( Determ )  THEN BEGIN
			pError^.WriteLine( pInput^.EchoLine ) ;
			pError^.Fatal( paMatrix^.ErrorMessageIs ) ;
		END
		ELSE BEGIN
      	Writeln( FOut, 'Determinant: ', Determ ) ;
		END ;
	END ;					{  InvertMatrix  }


PROCEDURE	TMatrixJugglerObject.MultiplyMatrices ;
	CONST
		ExpectedLine	= 'Expected line:  Multiply  Multiplicand  Multiplier  Product' ;
		MethodName		= 'MultiplyMatrices' ;
	VAR
		iA, iB, iC	: INTEGER;
		nra,nca	: WORD ;
		pMultiplicand	: PNamedMatrix ;
		pMultiplier	: PNamedMatrix ;
		pProduct		: PNamedMatrix ;
		sMultiplicand	: STRING ;
		sMultiplier	: STRING ;
		sProduct		: STRING ;
	BEGIN					{  MultiplyMatrices  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( sMultiplicand ) = '')
			OR  (GetToken( sMultiplier ) = '')
			OR  (GetToken( sProduct ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  Multiplicand exists?  }
		pMultiplicand	:= FindMatrix( sMultiplicand, iA ) ;
		IF  pMultiplicand = NIL  THEN BEGIN
			ExistError( sMultiplicand, MethodName ) ;
		END ;
		nra	:= pMultiplicand^.Rows ;
		nca	:= pMultiplicand^.Cols ;
							{  Multiplier exists?  }
		pMultiplier	:= FindMatrix( sMultiplier, iB ) ;
		IF  pMultiplier = NIL  THEN BEGIN
			ExistError( sMultiplier, MethodName ) ;
		END ;
							{  if Product exists, trash it  }
		pProduct	:= FindMatrix( sProduct, iC ) ;
		IF  pProduct <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( pProduct, Done ) ;
		END;
		pProduct := New( PNamedMatrix, Init( sProduct, nra, nca )) ;
		pMultiplicand^.Multiply( pMultiplier^, pProduct^ ) ;
		IF  pMultiplicand^.ErrorStatusIs <> 0  THEN BEGIN
			ShapeError( MethodName ) ;
		END ;
		pMatrices^.AtInsert( IC, pProduct ) ;
	END ;					{  MultiplyMatrices  }


PROCEDURE	TMatrixJugglerObject.PrintMatrix ;
	CONST
		ExpectedLine	= 'Expected line:  Print aMatrix' ;
		MethodName		= 'PrintMatrix' ;
	VAR
		iA	: INTEGER;
		paMatrix	: PNamedMatrix ;
		saMatrix	: STRING ;
	BEGIN					{  PrintMatrix  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( saMatrix ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  aMatrix exists?  }
		paMatrix	:= FindMatrix( saMatrix, iA ) ;
		IF  paMatrix = NIL  THEN BEGIN
			ExistError( saMatrix, MethodName ) ;
		END ;
		paMatrix^.Printit( FOut ) ;
	END ;					{  PrintMatrix  }


PROCEDURE	TMatrixJugglerObject.PrintTheCollection ;
	BEGIN					{  PrintTheCollection  }
		pMatrices^.Print( FOut ) ;
	END ;					{  PrintTheCollection  }


PROCEDURE	TMatrixJugglerObject.ReadMatrix;
	CONST
		ExpectedLine	= 'Expected line:  Read  aMatrix  nRows  nCols' ;
      MethodName		= 'ReadMatrix' ;
	VAR
		aName		: STRING ;
		aFloat	: FLOAT ;
		b 			: BOOLEAN ;
		iC			: INTEGER ;
		icode, limit : INTEGER ;
		pNM		: PNamedMatrix ;
		nr, nc	: INTEGER ;
	PROCEDURE	ReportError ;
		BEGIN					{  ReportError  }
			pError^.WriteLine( ExpectedLine );
			pError^.WriteLine( 'Found: ' + pInput^.EchoLine );
		END ;					{  ReportError  }
	BEGIN					{  ReadMatrix  }
		WITH  pInput^  DO BEGIN
			IF  GetToken( aName ) = ''  THEN BEGIN
         	SyntaxError( ExpectedLine, MethodName ) ;
			END ;
								{  if aName exists, trash it  }
			pNM	:= FindMatrix( aName, iC ) ;
			IF  pNM <> NIL  THEN BEGIN
				pMatrices^.AtDelete( iC ) ;
				Dispose( pNM, Done ) ;
			END;
			b := GetInt( nr, icode ) ;
			IF  NOT (b  AND  (icode=0))  THEN BEGIN
         	SyntaxError( ExpectedLine, MethodName ) ;
			END ;
			b := GetInt( nc, icode ) ;
			IF  NOT (b  AND  (icode=0))  THEN BEGIN
         	SyntaxError( ExpectedLine, MethodName ) ;
			END ;
			pNM	:= New( PNamedMatrix, Init( aName, WORD(nr), WORD(nc)) );
			IF  pNM = NIL  THEN BEGIN
				pError^.WriteLine( pInput^.EchoLine ) ;
            pError^.Fatal( 'Failed to instantiate the matrix.' ) ;
			END;
			FOR  nr := 1  TO  nr  DO BEGIN
				GetNonBlankLine ;
				FOR  nc := 1  TO  nc  DO BEGIN
            	b := GetFloat( aFloat, icode ) ;	
					IF  NOT (b  AND  (icode=0))  THEN BEGIN
						pError^.WriteLine( pInput^.EchoLine ) ;
						pError^.Fatal( 'Trouble reading row ' + IntToStr(nr) ) ;
					END ;
					pNM^.AssignTo( nr,nc, aFloat ) ;
				END ;
			END ;
			pMatrices^.Insert( pNM ) ;
		END ;
	END ;					{  ReadMatrix  }


PROCEDURE	TMatrixJugglerObject.ShapeError( aMethodName : STRING ) ;
	BEGIN					{  ShapeError  }
		pError^.WriteLine( pInput^.EchoLine );
		pError^.Fatal( 'Matrix shapes don''t match. - '+ aMethodName + '.' ) ;
	END ;					{  ShapeError  }


PROCEDURE	TMatrixJugglerObject.SubtractMatrices ;
	CONST
		ExpectedLine	= 'Expected line:  Subtract  Minuend  Subtrahend  Difference' ;
		MethodName		= 'SubtractMatrices' ;
	VAR
		iA, iB, iC	: INTEGER;
		nra,nca	: WORD ;
		pMinuend	: PNamedMatrix ;
		pSubtrahend	: PNamedMatrix ;
		pDifference		: PNamedMatrix ;
		sMinuend	: STRING ;
		sSubtrahend	: STRING ;
		sDifference		: STRING ;
	BEGIN					{  SubtractMatrices  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( sMinuend ) = '')
			OR  (GetToken( sSubtrahend ) = '')
			OR  (GetToken( sDifference ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  Minuend exists?  }
		pMinuend	:= FindMatrix( sMinuend, iA ) ;
		IF  pMinuend = NIL  THEN BEGIN
			ExistError( sMinuend, MethodName ) ;
		END ;
		nra	:= pMinuend^.Rows ;
		nca	:= pMinuend^.Cols ;
							{  Subtrahend exists?  }
		pSubtrahend	:= FindMatrix( sSubtrahend, iB ) ;
		IF  pSubtrahend = NIL  THEN BEGIN
			ExistError( sSubtrahend, MethodName ) ;
		END ;
							{  if Difference exists, trash it  }
		pDifference	:= FindMatrix( sDifference, iC ) ;
		IF  pDifference <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( pDifference, Done ) ;
		END;
		pDifference := New( PNamedMatrix, Init( sDifference, nra, nca )) ;
		pMinuend^.Subtract( pSubtrahend^, pDifference^ ) ;
		IF  pMinuend^.ErrorStatusIs <> 0  THEN BEGIN
			ShapeError( MethodName ) ;
		END ;
		pMatrices^.AtInsert( IC, pDifference ) ;
	END ;					{  SubtractMatrices  }


PROCEDURE	TMatrixJugglerObject.SyntaxError( ExpectedLine, aMethodName : STRING ) ;
	BEGIN					{  SyntaxError  }
		pError^.WriteLine( ExpectedLine );
		pError^.WriteLine( 'Found: ' + pInput^.EchoLine );
		pError^.Fatal( 'Bad syntax. - ' + aMethodName + '.' ) ;
	END ;					{  SyntaxError  }


PROCEDURE	TMatrixJugglerObject.TransposeMatrix ;
	CONST
		ExpectedLine	= 'Expected line:  Transpose Source Target' ;
		MethodName		= 'TransposeMatrix' ;
	VAR
		iA, iC	: INTEGER;
		nra,nca	: WORD ;
		pSource	: PNamedMatrix ;
		pTarget	: PNamedMatrix ;
		sSource	: STRING ;
		sTarget	: STRING ;
	BEGIN					{  TransposeMatrix  }
		WITH  pInput^  DO BEGIN		{  read & check syntax  }
			IF  (GetToken( sSource ) = '')
			OR  (GetToken( sTarget ) = '')  THEN BEGIN
				SyntaxError( ExpectedLine, MethodName ) ;
			END ;
		END ;
							{  Source exists?  }
		pSource	:= FindMatrix( sSource, iA ) ;
		IF  pSource = NIL  THEN BEGIN
			ExistError( sSource, MethodName ) ;
		END ;
		nra	:= pSource^.Rows ;
		nca	:= pSource^.Cols ;
							{  if Target exists, trash it  }
		pTarget	:= FindMatrix( sTarget, iC ) ;
		IF  pTarget <> NIL  THEN BEGIN
			pMatrices^.AtDelete( iC ) ;
			Dispose( pTarget, Done ) ;
		END;
		pTarget := New( PNamedMatrix, Init( sTarget, nra, nca )) ;
		pSource^.Transpose( pTarget^ );
		pMatrices^.AtInsert( IC, pTarget ) ;
	END ;					{  TransposeMatrix  }


{ BEGIN					{  MJO initialization  }
END .						{  MJO  }
